@@ -16,7 +16,7 @@
 #include "dviInt.h"
 
 #ifndef lint
-static char rcsid[] VAR_UNUSED = "$Id: fontTFM.c,v 1.2 2000/05/18 12:50:33 lingnau Exp $";
+static char rcsid[] VAR_UNUSED = "$Id: fontTFM.c,v 1.3 2001/07/26 09:15:31 anselm Exp $";
 #endif /* lint */
 
 /*
@@ -38,6 +38,8 @@ static int TfmLoad _ANSI_ARGS_((Dvi_Interp *dviInterp, Dvi_Font *dviFont));
 static Dvi_Glyph *TfmGlyph _ANSI_ARGS_((Dvi_Font *dviFont, S32 character,
 					S32 *tfmWidthPtr, S32 *pixelWidthPtr));
 static int TfmClose _ANSI_ARGS_((Dvi_Font *dviFont));
+static int TfmIterate _ANSI_ARGS_((Dvi_Font *dviFont,
+				  Dvi_FontIterateCallbackProc callback));
 
 /*
  * ------------------------------------------------------------------------
@@ -60,7 +62,7 @@ int
 Dvi_CreateFontType_TFM ()
 {
     return Dvi_CreateFontType(dvi_font_tfm, "tfm", TfmLoad, TfmGlyph,
-			      TfmClose);
+			      TfmClose, TfmIterate);
 }
 
 /*
@@ -290,3 +292,11 @@ TfmClose(dviFont)
     ckfree((char *)info);
     return TCL_OK;
 }
+
+static int
+TfmIterate(dviFont, callback)
+    Dvi_Font *dviFont;
+    Dvi_FontIterateCallbackProc callback;
+{
+    return 1;
+}
