@@ -12,7 +12,7 @@
 #include "dviInt.h"
 
 #ifndef lint
-static char rcsid[] VAR_UNUSED = "$Id: fontVF.c,v 1.1 1999/06/15 08:08:50 lingnau Exp $";
+static char rcsid[] VAR_UNUSED = "$Id: fontVF.c,v 1.2 2001/07/26 09:15:31 anselm Exp $";
 #endif /* lint */
 
 typedef struct VfCharInfo {
@@ -69,6 +69,8 @@ static U8 *OutputDviOp _ANSI_ARGS_((int op, U8 *outPtr, S32 value));
 static Dvi_Glyph *VfGlyph _ANSI_ARGS_((Dvi_Font *dviFont, S32 character,
 				       S32 *tfmWidthPtr, S32 *pixelWidthPtr));
 static int VfClose _ANSI_ARGS_((Dvi_Font *dviFont));
+static int VfIterate _ANSI_ARGS_((Dvi_Font *dviFont,
+				  Dvi_FontIterateCallbackProc callback));
 
 /*
  * ------------------------------------------------------------------------
@@ -90,7 +92,8 @@ static int VfClose _ANSI_ARGS_((Dvi_Font *dviFont));
 int
 Dvi_CreateFontType_VF ()
 {
-    return Dvi_CreateFontType(dvi_font_vf, "vf", VfLoad, VfGlyph, VfClose);
+    return Dvi_CreateFontType(dvi_font_vf, "vf", VfLoad, VfGlyph, VfClose,
+			      VfIterate);
 }
 
 /*
@@ -749,3 +752,11 @@ VfClose (dviFont)
 
     return TCL_OK;
 }
+
+static int
+VfIterate(dviFont, callback)
+    Dvi_Font *dviFont;
+    Dvi_FontIterateCallbackProc callback;
+{
+    return 1;
+}
