@@ -1,10 +1,19 @@
 # browser.tcl -- Make TkDVI browser windows.
-# Copyright  2000 Anselm Lingnau <lingnau@tm.informatik.uni-frankfurt.de>.
+# Copyright  2001 Anselm Lingnau <anselm@strathspey.org>
 # See file COPYING for conditions on use and distribution.
-# $Id: browser.tcl,v 1.12 2000/07/07 09:02:32 lingnau Exp $
+# $Id: browser.tcl,v 1.22 2001/08/10 11:34:59 anselm Exp $
 
 package provide tkdvi::browser 0.2
 
+# Read message catalogs just once for all TkDVI subpackages
+package require msgcat
+namespace eval ::tkdvi {
+    if {![info exists msgCatsLoaded]} {
+	::msgcat::mcload [file join [file dirname [info script]] msgs]
+	set msgCatsLoaded 1
+    }
+}
+
 namespace eval ::tkdvi::browser:: {
     variable Configure
     variable State
@@ -30,7 +39,10 @@ namespace eval ::tkdvi::browser:: {
 	measuretape {MeasureTape 1}
 	measureunit {MeasureUnit cm}
 	sourcespecials {SourceSpecials 0}
+	hushspecials {HushSpecials 0}
 	freehandcolor {FreehandColor red}
+	gamma {Gamma 1.2}
+	editor {Editor {emacsclient --no-wait +%l %f}}
     }
 
     foreach {option cv} [array get Options] {
@@ -39,6 +51,36 @@ namespace eval ::tkdvi::browser:: {
     }
 }
 
+proc ::tkdvi::browser::addHook {hook code} {
+    variable State
+    if {[info exists State(hooks,$hook)]} {
+	lappend State(hooks,$hook) $code
+    } else {
+	set State(hooks,$hook) $code
+    }
+}
+
+proc ::tkdvi::browser::addBinding {type key code} {
+    variable State
+    # puts stderr "Add binding: key = $key code = $code"
+    if {![info exists State(bindings,$type)]} {
+	set State(bindings,$type) {}
+    }
+    lappend State(bindings,$type) [list $key $code]
+}
+
+namespace eval ::tkdvi::browser {
+    frame .tkdvibrowserspecials -class TkDVIBrowser
+    foreach s {specials userspecials} {
+	foreach ss [option get .tkdvibrowserspecials $s Specials] {
+	    if {[catch { package require tkdvi::browser::special::$ss } msg]} {
+		puts stderr "$ss: $msg"
+	    }
+	}
+    }
+    destroy .tkdvibrowserspecials
+}
+
 proc ::tkdvi::browser::Defaults {name} {
     variable Configure
     variable State
@@ -59,7 +101,10 @@ proc ::tkdvi::browser::Defaults {name} {
 	    [namespace code [list Images $name -1]]
     trace variable Configure($name-shrink) w \
 	    [namespace code [list Images $name -1]]
-    
+
+    trace variable Configure($name-gamma) w \
+	    [namespace code [list Images $name -1]]
+
     trace variable Configure($name-titleformat) w \
 	    [namespace code [list TitleFormat $name]]
 
@@ -132,7 +177,7 @@ proc ::tkdvi::browser::browser {name args} {
 	    [::tkdvi::pagesel::pagesel $State($name-top).pg \
 	        -command [list ::$name gotopage %p] -background white]
 
-    set State($name-toolbar) [frame $State($name-top).tb]
+    set State($name-toolbar) [frame $State($name-top).tb -class Toolbar]
     entry $State($name-toolbar).e -width 15 -background white -justify center \
 	    -textvariable ::tkdvi::browser::State($name-pageNo) -state disabled
     set State($name-pageNoEntry) $State($name-toolbar).e
@@ -187,7 +232,7 @@ proc ::tkdvi::browser::browser {name args} {
 	    -ipadx 2 -ipady 2 -side left
 
     set State($name-menu) $State($name-top).m
-    ::tkdvi::menu::menubar $State($name-menu) $State($name-top) 
+    ::tkdvi::menu::menubar $name $State($name-menu) $State($name-top) 
 
     if {[catch "Opconfigure $name $args" result]} {
 	return -code error $result
@@ -264,11 +309,10 @@ proc ::tkdvi::browser::OpenFile {name args} {
     $State($name-code) configure \
 	    -reloadcommand [list ::tkdvi::browser::ReloadFinish $name]
 
-    # The `Bookmarks ...' menu item will be reenabled later if there are
-    # actually any bookmarks in the file.
-
-    foreach {menu idx} [::tkdvi::menu::finditem "Bookmarks *"] {
-	$menu entryconfigure $idx -state disabled
+    if {[info exists State(hooks,open)]} {
+	foreach h $State(hooks,open) {
+	    eval $h $name
+	}
     }
 
     TitleFormat $name
@@ -376,8 +420,13 @@ proc ::tkdvi::browser::CreateDviImage {name canvas key x y shrink page} {
     variable Configure
     variable State
 
+    if {[info exists State($name-$key,i)] \
+	    && [lsearch [image names] $State($name-$key,i)] >= 0} {
+	image delete $State($name-$key,i)
+    }
     set State($name-$key,i) [::image create dvi -file $State($name-code) \
 	    -shrink $shrink -size $Configure($name-size) \
+	    -gamma $Configure($name-gamma) \
 	    -specialcommand \
 	        [namespace code [list SpecialCmd $name $key %x %y %c %s]] \
 	    -precommand [namespace code [list PreCmd $name $key %p]] \
@@ -486,7 +535,7 @@ proc ::tkdvi::browser::B3Pressed {name state item shrink overviewMode x y X Y} {
 	} elseif {$state & 4} {	# Control
 	    PostAlignment $name $x $y
 	} elseif {$state & 64} {	# Meta
-	    SrcSGo $name $x $y
+	    srcspec::go $name $x $y
 	}
     }
 }
@@ -526,50 +575,41 @@ proc ::tkdvi::browser::Canvasbindings {name} {
 
     set canvas $State($name-canvas)
 
+    ::tkdvi::menu::accelerators $canvas $State($name-menu)
+
     # Navigation
     bind $canvas <Left> [list $canvas xview scroll -1 pages]
     bind $canvas <Right> [list $canvas xview scroll 1 pages]
     bind $canvas <Down> [list $canvas yview scroll 1 pages]
     bind $canvas <Up> [list $canvas yview scroll -1 pages]
     
-    bind $canvas <minus> [list ::$name nextpage -1]
     bind $canvas <p> [list ::$name nextpage -1]
     bind $canvas <b> [list ::$name nextpage -1]
-    bind $canvas <plus> [list ::$name nextpage 1]
     bind $canvas <n> [list ::$name nextpage 1]
     bind $canvas <asterisk> [list ::$name nextpage 10]
     bind $canvas <underscore> [list ::$name nextpage -10]
     bind $canvas <space> [list ::$name nextpage 1 1]
 
-    bind $canvas <Home> [list ::$name gotopage 0]
-    bind $canvas <End> [list ::$name gotopage \
-	    [expr {[$State($name-code) info pages]-1}]]
-
     bind $canvas <g> [list ::$name gotopage]
 
+    # Mode changes
+    bind $canvas <o> \
+	    [list set ::tkdvi::browser::Configure($name-mode) overview]
+    bind $canvas <s> [list set ::tkdvi::browser::Configure($name-mode) single]
+    bind $canvas <d> [list set ::tkdvi::browser::Configure($name-mode) spread]
+
+    bind $canvas <f> [namespace code \
+	    [list ToggleConfigureVariable $name fullscreen]]
+
     # Scanning
     bind $canvas <ButtonPress-2> [namespace code [list StartScan $name %x %y]]
     bind $canvas <B2-Motion> [namespace code [list ContinueScan $name %x %y]]
     bind $canvas <ButtonRelease-2> [namespace code [list EndScan $name]]
 
     # Extras
-    bind $canvas <q> { exit }
-    bind $canvas <c> [list ::$name close]
-    bind $canvas <r> [list ::$name reload]
-    bind $canvas <w> [list ::tkdvi::cloneWindow $name]
-
-    bind $canvas <o> [list ::$name configure -mode overview]
-    bind $canvas <s> [list ::$name configure -mode single]
-    bind $canvas <d> [list ::$name configure -mode spread]
-
-    bind $canvas <f> \
-	    [namespace code [list ToggleConfigureVariable $name fullscreen]]
-
     bind $canvas <C> [namespace code [list FreehandClear $name]]
     bind $canvas <Delete> [namespace code [list FreehandDeleteItem $name]]
 
-    bind $canvas <z> [namespace code [list SrcSDump $name]]
-
     foreach i {1 2 3 4 5 6 7 8 9} {
 	bind $canvas <Key-$i> [namespace code [list NumArg $name $i]]
 	bind $canvas <Control-Key-$i> [list ::$name configure -shrink $i]
@@ -577,29 +617,28 @@ proc ::tkdvi::browser::Canvasbindings {name} {
     bind $canvas <Key-0> [namespace code [list NumArg $name 0]]
     bind $canvas <Control-Key-0> [list ::$name configure -shrink 10]
 
-    bind $canvas <Control-plus> \
-	    [list incr ::tkdvi::browser::Configure($name-shrink) -1]
-    bind $canvas <Control-minus> \
-	    [list incr ::tkdvi::browser::Configure($name-shrink)]
-
-    foreach {k v} {m menubar t toolbar p pageselector} {
-	bind $canvas <Control-Key-$k> \
-	    [namespace code [list ToggleConfigureVariable $name $v]]
-    }
-
     bind $canvas <Enter> [list ::focus $canvas]
 
+    if {[info exists State(bindings,canvas)]} {
+	foreach b $State(bindings,canvas) {
+	    foreach {key code} $b break
+	    regsub %T $code $name code
+	    regsub %C $code $canvas code
+	    bind $canvas $key $code
+	}
+    }
+
     bind $State($name-top) <Expose> [namespace code [list ReloadCheck $name]]
 
-    bind $State($name-top) <Destroy> [namespace code [list DestroyCheck $name %W]]
+    bind $State($name-top) <Destroy> \
+	    [namespace code [list DestroyCheck $name %W]]
 }
 
 proc ::tkdvi::browser::DestroyCheck {name widget} {
     variable State
     variable lastCloseExits
 
-    if {$lastCloseExits == 0 \
-	    || [string compare $State($name-top) $widget] != 0} {
+    if {$lastCloseExits == 0} {
 	return
     }
     foreach w [winfo children .] {
@@ -678,16 +717,66 @@ proc ::tkdvi::browser::SetPageNoDisplay {name pageNo {currLayer 0} {layers 0}} {
     variable State
     if {$layers == 0 || $currLayer == 9999} {
 	set State($name-pageNo) \
-		[format {%s of %d} [expr {$pageNo+1}] \
+		[::msgcat::mc {%s of %d} [expr {$pageNo+1}] \
 		[$State($name-code) info pages]]
     } else {
 	set State($name-pageNo) \
-		[format {%s of %d [%d/%d]} [expr {$pageNo+1}] \
+		[::msgcat::mc {%s of %d [%d/%d]} [expr {$pageNo+1}] \
 		[$State($name-code) info pages] [expr {$currLayer+1}] $layers]
     }
     $State($name-pageselector) current $pageNo
 }
 
+proc ::tkdvi::browser::GotoPageDialog {name} {
+    variable State
+
+    if {![info exists State($name-gpdialog)]} {
+	set top [toplevel $State($name-top).gp -class GotoPageDialog]
+	wm title $top [::msgcat::mc "Go To"]
+	label $top.l -text [::msgcat::mc "Go to page:"]
+	if {[llength [info command ::spinbox]]} { # Tk 8.4
+	    spinbox $top.p -wrap true \
+		    -values [$State($name-code) info pagenumbers]
+	} else {
+	    entry $top.p
+	}
+	$top.p configure -background white -foreground black -width 10
+	bind $top.p <Return> [namespace code [list GotoPageDialogGoto $name]]
+	frame $top.sep -relief sunken -borderwidth 1 -height 2
+	frame $top.buttons
+	button $top.buttons.ok -text [::msgcat::mc OK] \
+		-command [namespace code [list GotoPageDialogGoto $name]]
+	button $top.buttons.cancel -text [::msgcat::mc Cancel] \
+		-command [namespace code [list GotoPageDialogCancel $name]]
+	pack $top.buttons.ok $top.buttons.cancel -side left -fill x
+
+	grid $top.l $top.p -sticky we -padx 5 -pady 5
+	grid $top.sep -columnspan 2 -sticky we -pady 3
+	grid $top.buttons -columnspan 2
+
+	set State($name-gpdialog) $top
+    } else {
+	set top $State($name-gpdialog)
+	if {[string compare [winfo class $top.p] Spinbox] == 0} {
+	    $top.p -values [$State($name-code) info pagenumbers]
+	}
+	wm deiconify $top
+    }
+}
+
+proc ::tkdvi::browser::GotoPageDialogGoto {name} {
+    variable State
+    set top $State($name-gpdialog)
+    $name gotopage [$top.p get]
+    wm withdraw $top
+}
+
+proc ::tkdvi::browser::GotoPageDialogCancel {name} {
+    variable State
+    set top $State($name-gpdialog)
+    wm withdraw $top
+}
+
 # Method dispatcher
 
 proc ::tkdvi::browser::Methods {name method argList} {
@@ -775,6 +864,9 @@ proc ::tkdvi::browser::Opnextpage {name {offset 1} {nextLayer 0}} {
     variable State
     variable Configure
 
+    set cursor [$State($name-canvas) cget -cursor]
+    $State($name-canvas) configure -cursor watch
+
     if {abs($offset) == 1} {
 	set offset [expr {$offset<0?-[CheckNumArg $name]:[CheckNumArg $name]}]
     }
@@ -827,12 +919,17 @@ proc ::tkdvi::browser::Opnextpage {name {offset 1} {nextLayer 0}} {
     update idletasks
     set layers [$State($name-dvi0,i) layers]
     SetPageNoDisplay $name $State($name-dvi0,p) $currLayer $layers
+
+    $State($name-canvas) configure -cursor $cursor
 }
 
 proc ::tkdvi::browser::Opgotopage {name {page -1}} {
     variable State
     variable Configure
 
+    set cursor [$State($name-canvas) cget -cursor]
+    $State($name-canvas) configure -cursor watch
+
     set lastPage [expr {[$State($name-code) info pages] - 1}]
 
     if {[string compare $page last] == 0} {
@@ -856,6 +953,8 @@ proc ::tkdvi::browser::Opgotopage {name {page -1}} {
 	incr i
     }
     SetPageNoDisplay $name $State($name-dvi0,p)
+
+    $State($name-canvas) configure -cursor $cursor
 }
 
 proc ::tkdvi::browser::Opprint {name args} {
@@ -879,11 +978,11 @@ proc ::tkdvi::browser::Opopen {name {file {}} args} {
     variable State
 
     if {[string length $file] == 0} {
-	set types {
-	    {{DVI Files} {.dvi}}
-	    {{All Files} *}
-	}
-	set file [tk_getOpenFile -filetypes $types -title "Open DVI File"]
+	set types [list \
+		[list [::msgcat::mc {DVI Files}] {.dvi}] \
+		[list [::msgcat::mc {All Files}] *]]
+	set file [tk_getOpenFile -filetypes $types \
+		-title [::msgcat::mc "Open DVI File"]]
     }
     if {[string length $file] > 0} {
 	if {[catch {set f [::dvi::code create -file $file]} msg]} {
@@ -900,21 +999,37 @@ proc ::tkdvi::browser::Opclose {name args} {
     variable Configure
     variable State
 
-    destroy $State($name-top)
+    if {[info exists State(hooks,close)]} {
+	foreach h $State(hooks,close) {
+	    eval $h $name
+	}
+    }
+
+    set top $State($name-top)
     rename $name {}
-    foreach n [array names Configure($name-*)] {
-	unset $n
+    foreach n [array names Configure $name-*] {
+	unset Configure($n)
     }
-    foreach n [array names State($name-*)] {
-	unset $n
+    foreach dvi $State($name-dvis) {
+	# puts stderr "Delete: $State($name-$dvi,i)"
+	image delete $State($name-$dvi,i)
     }
+    foreach n [array names State $name-*] {
+	unset State($n)
+    }
+    destroy $top
 }
 
 proc ::tkdvi::browser::Opreload {name args} {
     variable Configure
     variable State
 
+    set cursor [$State($name-canvas) cget -cursor]
+    $State($name-canvas) configure -cursor watch
+
     $State($name-code) reload
+
+    $State($name-canvas) configure -cursor $cursor
 }
 
 # Special commands
@@ -922,78 +1037,43 @@ proc ::tkdvi::browser::Opreload {name args} {
 proc ::tkdvi::browser::PreCmd {name key page} {
     variable State
     $State($name-canvas) move p=$page -100000 -100000
-    set State($name-srcYs) {}
+    if {[info exists State(hooks,predisplay)]} {
+	foreach h $State(hooks,predisplay) {
+	    eval $h $name
+	}
+    }
     return 0
 }
 proc ::tkdvi::browser::PostCmd {name key page} {
     variable State
+
+    if {[info exists State(hooks,postdisplay)]} {
+	foreach h $State(hooks,postdisplay) {
+	    eval $h $name
+	}
+    }
     $State($name-canvas) move p=$page 100000 100000
-    foreach s [array names State $name-srcY,*] { unset $s }
-    foreach s [array names State $name-src,*] { unset $s }
-    unset State($name-srcYs)
     return 0
 }
+
+namespace eval ::tkdvi::browser::special {}
+
+proc ::tkdvi::browser::special::html {name key x y cookie s} {
+}
+
 proc ::tkdvi::browser::SpecialCmd {name key x y cookie s} {
     variable State
     variable Configure
 
-    if {$Configure($name-sourcespecials) \
-	    && [regexp {src:(\d+) *(.*)} $s junk lineno fileName]} {
-	if {![info exists State($name-src,$cookie)]} {
-	    puts stderr "Source special: $x $y $fileName $lineno"
-	    $State($name-canvas) create rectangle \
-		    [expr {$x-1}] [expr {$y-1}] [expr {$x+1}] [expr {$y+1}] \
-		    -outline red -tags [list src "${fileName}#${lineno}"]
-	    lappend State($name-srcY,$y) $x
-	    set State($name-src,$cookie) 1
-	}
-    } elseif {[scan $s {info %[^ ]} key] == 1} {
-	set key [lindex $s 1]
-	set State($name-info,$key) \
-		[string range $s [expr {6+[string length $key]}] end]
-    } elseif {!$State($name-bmFinalized) \
-	    && [string compare $s "bookmark begin"] == 0} {
-	BookmarkBegin $name
-    } elseif {!$State($name-bmFinalized) \
-	    && [string compare $s "bookmark end"] == 0} {
-	set State($name-bmFinalized) 1
-	BookmarkEnd $name
-    } elseif {!$State($name-bmFinalized) \
-	    && [regexp {bookmark add ([0-9]+) ([^ ]+) (.*)} \
-	    $s junk level mark title]} {
-	BookmarkAdd $name $level $mark $title
-    } elseif {[scan $s {example %s %f %f} prog wd ht] == 3} {
-	if {[$State($name-$key,i) cget -shrink] > $Configure($name-shrink)} {
-	    return
-	}
-
-	if {![info exists State($name-tags,$cookie)]} {
-	    set State($name-tags,$cookie) t$cookie
-	    set wpx [expr \
-		    {[dvi::pixels [$State($name-$key,i) cget -xresolution] \
-		        ${wd}pt] / $State($name-$key,s)}]
-	    set wpy [expr \
-		    {[dvi::pixels [$State($name-$key,i) cget -yresolution] \
-		        ${ht}pt] / $State($name-$key,s)}]
-	    set file [file join [file dirname \
-		    [$State($name-code) info filename]] $prog]
-	    if {[catch {uplevel \#0 [list source $file]} msg]} {
-		puts stderr $msg
+    if {[scan $s {%[A-Za-z0-9]} cmd] == 1} {
+	if {[catch {namespace eval special \
+		$cmd $name $key $x $y $cookie [list $s]} msg]} {
+	    if {!$Configure($name-hushspecials)} {
+		puts stderr "special: $msg"
 	    }
-	    set f [frame $State($name-canvas).$State($name-tags,$cookie) \
-		    -width $wpx -height $wpy \
-		    -background [$State($name-canvas) cget -background]]
-	    pack propagate $f false
-	    set fn [file rootname $prog]_init
-	    if {[catch {uplevel \#0 $fn $f $wpx $wpy} msg]} {
-		puts stderr $msg
-	    }
-	    $State($name-canvas) create window $x $y -anchor nw -window $f \
-		    -tags [list $State($name-tags,$cookie) \
-		        p=[$State($name-$key,i) page]]
-	    $State($name-canvas) raise $State($name-tags,$cookie)
-	    set State($name-frames,$cookie) $f
 	}
+    } elseif {!$Configure($name-hushspecials)} {
+	puts stderr [::msgcat::mc "special: unrecognized: %s" $s]
     }
 }
 
@@ -1121,15 +1201,22 @@ proc ::tkdvi::browser::FreehandDeleteItem {name} {
 # Measuring ruler
 
 namespace eval ::tkdvi::browser {
-    variable UnitNameList
     variable UnitNames
     variable UnitFmts
-    set UnitNameList {in cm mm pt bp pc dd cc sp px}
     if {![info exists UnitNames]} {
-	set UnitNames [list {in Inch} {cm Centimetres} {mm Millimetres} \
-	    {pt {Printer's points (1/72.27in)}} {bp {Big points (1/72in)}} \
-	    {pc Picas} {dd {Didot points}} {cc {Ciceros (12dd)}} \
-	    {sp {Scaled points (1/65536pt)}} {px {Device pixels}}]
+	foreach {u n} [list \
+		in [::msgcat::mc Inch] \
+		cm [::msgcat::mc Centimetres] \
+		mm [::msgcat::mc Millimetres] \
+		pt [::msgcat::mc {Printer's points (1/72.27in)}] \
+		bp [::msgcat::mc {Big points (1/72in)}] \
+		pc [::msgcat::mc Picas] \
+		dd [::msgcat::mc {Didot points}] \
+		cc [::msgcat::mc {Ciceros (12dd)}] \
+		sp [::msgcat::mc {Scaled points (1/65536pt)}] \
+		px [::msgcat::mc {Device pixels}]] {
+	    lappend UnitNames [list $u [format "%s - %s" $u $n]]
+	}
     }
     if {![info exists UnitFmts]} {
 	array set UnitFmts {
@@ -1271,43 +1358,53 @@ proc ::tkdvi::browser::GeneralPrefs {name} {
     variable State
     variable Dialog
     variable DisplayModeKeys
-    variable UnitNameList
     variable UnitNames
 
     set p $State($name-top).prefs
     set State($name-prefs) $p
 
     toplevel $p
-    wm title $p "General Preferences"
+    wm title $p [::msgcat::mc "General Preferences"]
     set row 0
     
     set Dialog($name-defaultmode) [option get $name mode Mode]
     set Dialog($name-measureunit) $Configure($name-measureunit)
+    set Dialog($name-gamma) $Configure($name-gamma)
 
-    label $p.layout-l -text {Default Display Mode: } -anchor e
+    label $p.layout-l -text [::msgcat::mc {Default Display Mode: }] \
+	    -anchor e
     eval OptionMenu $p.layout-m \
 	    tkdvi::browser::Dialog($name-defaultmode) $DisplayModeKeys
     grid $p.layout-l -row $row -column 0 -sticky e -ipadx 5 -ipady 5
     grid $p.layout-m -row $row -column 1 -sticky we -padx 2 -pady 2
     incr row
 
-    label $p.unit-l -text {Measurement unit: } -anchor e
+    label $p.unit-l -text [::msgcat::mc {Measurement unit: }] -anchor e
     eval OptionMenu $p.unit-m \
 	    tkdvi::browser::Dialog($name-measureunit) $UnitNames
     grid $p.unit-l -row $row -column 0 -sticky e -ipadx 5 -ipady 5
     grid $p.unit-m -row $row -column 1 -sticky we -padx 2 -pady 2
     incr row
 
+    label $p.gamma-l -text [::msgcat::mc {Gamma value:}] -anchor e
+    scale $p.gamma-s -orient horizontal -length 80 -resolution 0.1 \
+	    -showvalue true -from 0 -to 5 -tickinterval 0 \
+	    -variable tkdvi::browser::Dialog($name-gamma)
+    grid $p.gamma-l -row $row -column 0 -sticky e -ipadx 5 -ipady 5
+    grid $p.gamma-s -row $row -column 1 -sticky we -padx 2 -pady 2
+    incr row
+
     frame $p.sep -relief sunken -borderwidth 1 -height 2
     grid $p.sep -row $row -column 0 -columnspan 2 -sticky we
     incr row
 
     frame $p.buttons
-    button $p.buttons.ok -text OK \
+    button $p.buttons.ok -text [::msgcat::mc OK] \
 	    -command [namespace code [list GPApply $name 1]]
-    button $p.buttons.apply -text Apply \
+    button $p.buttons.apply -text [::msgcat::mc Apply] \
 	    -command [namespace code [list GPApply $name]]
-    button $p.buttons.cancel -text Cancel -command [list destroy $p]
+    button $p.buttons.cancel -text [::msgcat::mc Cancel] \
+	    -command [list destroy $p]
     pack $p.buttons.ok $p.buttons.apply $p.buttons.cancel \
 	    -side left -fill x -padx 10 -pady 5
     grid $p.buttons -row $row -column 0 -columnspan 2
@@ -1319,6 +1416,7 @@ proc tkdvi::browser::GPApply {name {quit 0}} {
     variable Dialog
 
     set Configure($name-measureunit) $Dialog($name-measureunit)
+    set Configure($name-gamma) $Dialog($name-gamma)
     option add *TkDVIBrowser.mode $Dialog($name-defaultmode) interactive
 
     if {$quit} {
@@ -1326,210 +1424,6 @@ proc tkdvi::browser::GPApply {name {quit 0}} {
     }
 }
 
-# General Info ( la PDF)
-
-proc tkdvi::browser::InfoGeneral {name} {
-    variable State
-
-    set State($name-info,file) [$State($name-code) info filename]
-    set State($name-info,comment) [$State($name-code) info comment]
-    set State($name-info,modified) \
-	    [clock format [file mtime $State($name-info,file)]]
-    set State($name-info,size) [file size $State($name-info,file)]
-
-    set keys {file title subject author keywords producer modified size}
-
-    destroy $State($name-top).info
-    set t [toplevel $State($name-top).info]
-    wm title $t [format {Info: %s} \
-	    [file tail [file rootname $State($name-info,file)]]]
-
-    set row 0
-    foreach kk [concat $keys [array names State $name-info,*]] {
-	regsub "^$name-info," $kk {} k
-	if {[info exists seen($k)] || ![info exists State($name-info,$k)]} {
-	    continue
-	}
-	if {[catch {set title [string totitle $k]}]} {
-	    set title $k
-	}
-	label $t.k$row -anchor e -text "$title:"
-	label $t.v$row -anchor w -text $State($name-info,$k)
-	grid $t.k$row -row $row -column 0 -sticky we -pady 3
-	grid $t.v$row -row $row -column 1 -sticky we -pady 3
-	set seen($k) 1
-	incr row
-    }
-
-    frame $t.sep -relief sunken -borderwidth 1 -height 2
-    grid $t.sep -row $row -column 0 -columnspan 2 -sticky we -pady 3
-    incr row
-
-    frame $t.button
-    button $t.button.ok -text OK -command [list destroy $t]
-    pack $t.button.ok -pady 5
-    grid $t.button -row $row -column 0 -columnspan 2 -sticky we
-}
-
-# Font Info (not really)
-
-proc ::tkdvi::browser::InfoFonts {name} {
-    variable State
-
-    foreach f [$State($name-dvi0,i) fonts] {
-	foreach {num info} $f break
-	# puts stderr "$num => $info"
-	puts stderr [format "%d: %s %f" $num [lindex $info 0] [expr {double([lindex $info 2])/double([lindex $info 3])}]]
-    }
-}
-
-# Bookmarks
-
-proc ::tkdvi::browser::BookmarkBegin {name} {
-    variable State
-
-    if {[info exists State($name-bookmarks)]} {
-	destroy $State($name-bookmarks)
-    }
-    set State($name-bookmarks) [toplevel $State($name-top).bookmarks]
-    wm withdraw $State($name-bookmarks)
-    wm title $State($name-bookmarks) \
-	    [format {Bookmarks: %s} [file tail \
-	    [file rootname [$State($name-code) info filename]]]]
-    set t [text $State($name-bookmarks).t -width 40 -height 20 \
-	    -cursor arrow -background white -font {Helvetica 12} \
-	    -yscrollcommand [list $State($name-bookmarks).sy set]]
-    $t tag configure level1 -lmargin1 0 -lmargin2 30
-    $t tag configure level2 -lmargin1 20 -lmargin2 50
-    $t tag configure level3 -lmargin1 40 -lmargin2 70
-    $t tag configure level4 -lmargin1 60 -lmargin2 90
-    $t tag configure level5 -lmargin1 80 -lmargin2 110
-    bindtags $t [list bookmark Text]
-    bind bookmark <1> [namespace code [list BookmarkGoto $name]]
-    grid $t -row 0 -column 0 -sticky nwse
-    scrollbar $State($name-bookmarks).sy -orient vertical \
-	    -command [list $t yview]
-    grid $State($name-bookmarks).sy -row 0 -column 1 -sticky ns
-    grid rowconfigure $State($name-bookmarks) 0 -weight 1
-    grid columnconfigure $State($name-bookmarks) 0 -weight 1
-    frame $State($name-bookmarks).f
-    button $State($name-bookmarks).f.close -text Close \
-	    -command [list wm withdraw $State($name-bookmarks)]
-    pack $State($name-bookmarks).f.close
-    grid $State($name-bookmarks).f -row 1 -column 0 -columnspan 2
-}
-proc ::tkdvi::browser::BookmarkEnd {name} {
-    variable State
-    $State($name-bookmarks).t configure -state disabled
-#    if {[info exists State($name-bookmarkMenu)]} {
-#	foreach {menu idx} $State($name-bookmarkMenu) break
-#	$menu entryconfigure $idx -state normal
-#    }
-    foreach {menu idx} [::tkdvi::menu::finditem "Bookmarks *"] {
-	$menu entryconfigure $idx -state normal
-    }
-}
-proc ::tkdvi::browser::BookmarkAdd {name level mark title} {
-    variable State
-    regsub -all {\\([0-7][0-7][0-7])} $title {[format "%c" 0\1]} title
-    regsub -all {\\\(} $title {(} title
-    regsub -all {\\\)} $title {)} title
-    set title [subst -nobackslashes -novariables $title]
-    $State($name-bookmarks).t insert end \
-	    $title [list level$level m-$mark] "\n"
-}
-proc ::tkdvi::browser::BookmarkGoto {name} {
-    variable State
-    foreach tag [$State($name-bookmarks).t tag names current] {
-	if {[string match m-* $tag]} {
-	    set anchor [string range $tag 2 end]
-	    set page [$State($name-code) findanchor $anchor]
-	    if {$page >= 0} {
-		::$name gotopage $page
-	    }
-	}
-    }
-}
-proc ::tkdvi::browser::BookmarkShow {name} {
-    variable State
-    wm deiconify $State($name-bookmarks)
-}
-
-# Source specials
-
-proc ::tkdvi::browser::SrcSDump {name} {
-    variable State
-    puts stderr "Source specials:"
-    set ys [array names State $name-srcY,*]
-    foreach y [lsort $ys] {
-	puts stderr "$y => $State($y)"
-    }
-}
-proc ::tkdvi::browser::SrcSGo {name x y} {
-    variable Configure
-    variable State
-
-    if {!$Configure($name-sourcespecials)} {
-	return
-    }
-
-    set x [$State($name-canvas) canvasx $x]
-    set y [$State($name-canvas) canvasy $y]
-    if {[llength $State($name-srcYs)] == 0} {
-	set ll {}
-	set ys [array names State $name-srcY,*]
-	if {[llength $ys] == 0} {
-	    return
-	}
-	foreach yy $ys {
-	    lappend ll [string range $yy [expr {[string last , $yy]+1}] end]
-	}
-	set State($name-srcYs) [lsort -integer $ll]
-	lappend State($name-srcYs) 99999999
-    }
-
-    # Look for the current line (coordinate yyy where mouse y coordinate
-    # is less than yyy but greater than y coordinate of previous line).
-    # Check for source special on that line but to the left of mouse x
-    # coordinate; if there isn't one, check previous line with very large
-    # hypothetical mouse x coordinate. BUG: Doesn't seem to work well for
-    # multi-column documents.
-
-    for {set i 0} {$i < [expr {[llength $State($name-srcYs)]-1}]} {incr i} {
-	if {[lindex $State($name-srcYs) $i] > $y} break
-    }
-    while {$i > 0} {
-	set yyy [lindex $State($name-srcYs) $i]
-	if {$yyy < 99999999} {
-	    puts stderr "Likely Y: $yyy"
-	    set ys [concat -1 $State($name-srcY,$yyy) 99999999]
-	    for {set j 0} {$j < [expr {[llength $ys]}]} {incr j} {
-		if {[lindex $ys $j] > $x} break
-	    }
-	    if {[lindex $ys [expr {$j-1}]] >= 0} {
-		set xxx [lindex $ys [expr {$j-1}]]
-		break
-	    }
-	}
-	set x 99999998
-	incr i -1
-    }
-    puts stderr "Likely X: $xxx"
-
-    # Find appropriate file name and line number and emit command to
-    # go there.
-
-    foreach item [$State($name-canvas) find withtag src] {
-	foreach {ix iy} [$State($name-canvas) coords $item] break
-	set ix [expr {round($ix)+1}]
-	set iy [expr {round($iy)+1}]
-	if {$ix  == $xxx && $iy == $yyy} {
-	    puts stderr [$State($name-canvas) itemcget $item -tags]
-	    break
-	}
-    }
-}
-	
 # About box
 
 proc ::tkdvi::browser::About {} {
@@ -1540,16 +1434,21 @@ proc ::tkdvi::browser::About {} {
     set i [image create photo -file [file join $tkdvi(tcllib) tkdvi.gif]]
     label $c.icon -image $i
     pack $c.icon -padx 5 -pady 5 -side left
-    set text "TkDVI, a Tcl/Tk DVI Previewer\n"
-    append text "Version [tkdvi::version]\n"
-    append text "Copyright  2000 Anselm Lingnau\n\n"
+    set text [::msgcat::mc "TkDVI, a Tcl/Tk DVI Previewer"]
+    append text \n
+    append text [::msgcat::mc "Version %s" [tkdvi::version]]
+    append text \n
+    append text [::msgcat::mc "Copyright  2000 Anselm Lingnau"]
+    append text \n\n
     append text "Tcl/Tk [info patchlevel]"
     if {[info exists tcl_platform(os)]} {
-	append text " on $tcl_platform(os) $tcl_platform(osVersion)"
-	append text " ($tcl_platform(machine))"
+	append text [::msgcat::mc " on %s %s (%s)" \
+		$tcl_platform(os) $tcl_platform(osVersion) \
+		$tcl_platform(machine)]
     }
-    append text "\n\nFor information, see\n"
-    append text "http://tm.informatik.uni-frankfurt.de/~lingnau/tkdvi/"
+    append text \n\n
+    append text [::msgcat::mc "For information, see"]
+    append text "\nhttp://tkdvi.sourceforge.net/"
     message $c.msg -aspect 500 -text $text
     pack $c.msg -padx 5 -pady 5 -side right -expand yes -fill both
     button $b.ok -text OK -command {set aboutOK 1}
